@@ -1,5 +1,5 @@
 /**
- *   (c) 2001 Nathan Hjelm <hjelmn@unm.edu>
+ *   (c) 2001-2004 Nathan Hjelm <hjelmn@users.sourceforge.net>
  *   v1.0.6 bsd.c
  *   
  *   Functions that are possibly missing.
@@ -51,26 +51,3 @@ char *basename(char *x){
 }
 
 #endif
-
-#if !defined(HAVE_MEMMEM)
-
-/* define memmem */
-void *memmem(void *haystack, size_t haystack_size, void *needle, size_t needle_size) {
-    char *tmp = (char *)haystack;
-
-    if (!haystack)
-	return NULL;
-
-    /* for proper implimentation */
-    if (!needle)
-	return (haystack - 1);
-    
-    for (tmp = haystack ; ((int)tmp + needle_size) <= ((int)haystack + haystack_size) ; tmp++)
-      if (memcmp(tmp, needle, needle_size) == 0)
-	return tmp;
-    
-    /* not found */
-    return NULL;
-}
-
-#endif
