@@ -1,9 +1,9 @@
 /**
- *   (c) 2003 Nathan Hjelm <hjelmn@unm.edu>
+ *   (c) 2001-2004 Nathan Hjelm <hjelmn@users.sourceforge.net>
  *   v1.4.1c rio.c
  *   
  *   c version of librioutil
- *   all sources are c style bsd or gnu (c-set-style in emacs)
+ *   all sources are c style gnu (c-set-style in emacs)
  *   
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library Public License as published by
@@ -87,7 +87,8 @@ rios_t *open_rio (rios_t *rio, int number, int debug, int fill_structures) {
 	   "open_rio: creating new rio instance. device: 0x%08x\n", number);
   
   if (debug) {
-    rio_log (rio, 0, "open_rio: setting usb driver verbosity level to %i\n", debug);
+    rio_log (rio, 0, "open_rio: setting usb driver verbosity level to %i\n",
+	     debug);
 
     usb_setdebug(debug);
   }
@@ -101,7 +102,7 @@ rios_t *open_rio (rios_t *rio, int number, int debug, int fill_structures) {
     return NULL;
   }
   
-  if (set_time_rio (rio) != URIO_SUCCESS) {
+  if (set_time_rio (rio) != URIO_SUCCESS && fill_structures != 0) {
     close_rio (rio);
 
     return (rio->dev = NULL);
@@ -230,6 +231,7 @@ int get_flist_riomc (rios_t *rio, u_int8_t memory_unit, int *total_time,
 
     flist->num  = i;
     flist->inum = i;
+    flist->rio_num = file.file_no;
     strncpy(flist->artist, file.artist, 64);
     strncpy(flist->title, file.title, 64);
     strncpy(flist->album, file.album, 64);
@@ -257,6 +259,9 @@ int get_flist_riomc (rios_t *rio, u_int8_t memory_unit, int *total_time,
       flist->type = WAVE;
     else
       flist->type = OTHER;
+
+    if (return_generation_rio (rio) > 3)
+      memcpy (flist->sflags, file.unk1, 3);
     
     if (first == 1) {
       first = 0;
@@ -547,7 +552,7 @@ int return_generation_rio (rios_t *rio) {
       type == PSAPLAY || type == RIORIOT) 
     return 3;
   else if (type == RIOS10 || type == RIOS30 || type == RIOS35 ||
-	   type == RIOS50)
+	   type == RIOS50 || type == RIOS11)
     return 4;
   else if (type == RIOFUSE || type == RIOCHIBA || type == RIOCALI)
     return 5;
@@ -555,6 +560,10 @@ int return_generation_rio (rios_t *rio) {
     return -1;
 }
 
+float return_version_rio (rios_t *rio) {
+  return rio->info.version;
+}
+
 /*
   return_intrn_info_rio:
   BIG function that fills the rio_info structure.
@@ -765,7 +774,7 @@ static void sane_info_copy (rio_info_t *info, rio_prefs_t *prefs) {
   prefs->volume       = ((info->volume < 20)      ? info->volume       : 20);
 
   /* i don't think it would be a good idea to set this */
-  //prefs->playlist
+  /* prefs->playlist */
 
   if (strlen(info->name) > 0)
     strncpy(prefs->name, info->name, 16);
@@ -823,7 +832,7 @@ int format_mem_rio (rios_t *rio, u_int8_t memory_unit) {
   Update the firmware on a Rio. Function supports all rioutil supported players.
 */
 int update_rio (rios_t *rio, char *file_name) {
-  unsigned char fileBuffer[0x2000];
+  unsigned char fileBuffer[0x4000];
 
   struct stat statinfo;
   int size, x;
@@ -840,8 +849,11 @@ int update_rio (rios_t *rio, char *file_name) {
   if (try_lock_rio (rio) != 0)
     return ERIOBUSY;
 
-  rio_log (rio, 0, "Updating firmware of generation %d rio...\n", player_generation);
+  rio_log (rio, 0, "Updating firmware of generation %d rio...\n",
+	   player_generation);
   
+  rio_log (rio, 0, "Formatting internal memory\n");
+
   if ((ret = wake_rio(rio)) != URIO_SUCCESS)
     UNLOCK(ret);
 
@@ -947,6 +959,8 @@ int update_rio (rios_t *rio, char *file_name) {
   close(updtf);
 
   UNLOCK(URIO_SUCCESS);
+
+  return format_mem_rio (rio, 0);
 }
 
 int delete_dummy_hdr (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno) {
@@ -1068,6 +1082,7 @@ int download_file_rio (rios_t *rio, u_int8_t memory_unit,
   int leftover;
   int cr_dummy = -1;
   int mode = S_IRUSR | S_IWUSR | S_IROTH | S_IRGRP;
+  int block_size;
 
   int type = return_type_rio (rio);
   int player_generation = return_generation_rio (rio);
@@ -1093,8 +1108,9 @@ int download_file_rio (rios_t *rio, u_int8_t memory_unit,
     return -1;
   }
 
-  /* Ignore this section on Riot */
-  if (type != RIORIOT) {
+  /* Ignore this section on Riot and new flash players */
+  if (type != RIORIOT && return_generation_rio (rio) != 5 &&
+      (return_generation_rio (rio) != 4 && return_version_rio(rio) < 2.0)) {
     if (file.start == 0)
       UNLOCK(EUNSUPP);
 
@@ -1134,50 +1150,70 @@ int download_file_rio (rios_t *rio, u_int8_t memory_unit,
   else
     /* create a file with a user-specified name */
     downfd = creat((char *)fileName, mode);
+
+  if (return_generation_rio (rio) >= 4)
+    block_size = 0x4000;
+  else
+    block_size = 0x1000;
+
+  blocks = size/block_size + ((size % block_size) ? 1 : 0);
+
+  memset (downBuf, 0, block_size);
   
-  i=0;
-  blocks = size/0x1000;
-  
-  write_cksum_rio (rio, NULL, 0, "CRIODATA");
-  
-  read_block_rio(rio, NULL, 64);
-  
-  if (blocks)
-    for (;i<blocks;i++) {
-      read_block_rio (rio, downBuf, 0x1000);
+  for (i = 0 ; i < blocks ; i++) {
+    int read_size;
+    
+    if (rio->abort) {
+      abort_transfer_rio (rio);
+      rio->abort = 0;
       
       if (rio->progress)
-	rio->progress(i, blocks, rio->progress_ptr);
+	rio->progress(1, 1, rio->progress_ptr);
       
-      write(downfd, downBuf, 0x1000);
+      free(downBuf);
+      close(downfd);
+      UNLOCK(URIO_SUCCESS);
+    }
+    
+    /* the rio seems to expect checksum */
+    write_cksum_rio (rio, downBuf, block_size, "CRIODATA");
+    
+    if ((i+1)%4 == 0 || return_generation_rio(rio) >= 4) {
+      read_block_rio(rio, NULL, 64);
       
-      /* the rio seems to expect checksum */
-      write_cksum_rio (rio, downBuf, 0x1000, "CRIODATA");
-      
-      if ((i+1)%4 == 0) {
-	read_block_rio(rio, NULL, 64);
+      if (memcmp(rio->buffer, "SRIODONE", 8) == 0){
+	if (rio->progress)
+	  rio->progress(1, 1, rio->progress_ptr);
 	
-	if (memcmp(rio->buffer, "SRIODONE", 8) == 0){
-	  if (rio->progress)
-	    rio->progress(1, 1, rio->progress_ptr);
-	  
-	  free(downBuf);
-	  close(downfd);
-	  UNLOCK(URIO_SUCCESS);
-	}
+	free(downBuf);
+	close(downfd);
+	UNLOCK(URIO_SUCCESS);
       }
     }
-  
-  leftover = size - (i * 0x1000);
-  
-  if (leftover) {
-    read_block_rio (rio, downBuf, 0x1000);
-    write(downfd, downBuf, leftover);
     
-    write_cksum_rio (rio, downBuf, 0x1000, "CRIODATA");
-    i++;
+    if (size >= block_size)
+      read_size = block_size;
+    else
+      read_size = size;
+    
+    read_block_rio (rio, downBuf, block_size);
+    
+    if (rio->progress)
+      rio->progress(i, blocks, rio->progress_ptr);
+    
+    write(downfd, downBuf, read_size);
+    
+    size -= read_size;
+    
+    if (rio->debug > 0)
+      rio_log (rio, 0, "%08x bytes transferred, %08x to go\n", read_size, size);
   }
   
+  write_cksum_rio (rio, downBuf, block_size, "CRIODATA");
+  
+  if (return_generation_rio(rio) < 4)
+    read_block_rio(rio, NULL, 64);
+
   if (rio->progress)
     rio->progress(1, 1, rio->progress_ptr);
   
@@ -1187,16 +1223,18 @@ int download_file_rio (rios_t *rio, u_int8_t memory_unit,
      finish up a 16384 byte block (usually just zeros)
      I dont know exactly why it is needed
   */
-  for ( ; i % 4 ; i++) {
-    read_block_rio (rio, downBuf, 0x1000);
+  if (return_generation_rio (rio) < 4) {
+    for ( ; i % 4 ; i++) {
+      read_block_rio (rio, downBuf, block_size);
+      
+      write_cksum_rio (rio, downBuf, block_size, "CRIODATA");
+    }
     
-    write_cksum_rio (rio, downBuf, 0x1000, "CRIODATA");
-  }
-  
+    read_block_rio(rio, NULL, 64);
+  } 
+ 
   free(downBuf);
   
-  read_block_rio(rio, NULL, 64);
-  
   send_command_rio(rio, 0x60, 0, 0);
   
   /* If a dummy header was uploaded, clear it */
