@@ -1,5 +1,5 @@
 /**
- *   (c) 2003 Nathan Hjelm <hjelmn@unm.edu>
+ *   (c) 2001-2004 Nathan Hjelm <hjelmn@users.sourceforge.net>
  *   v1.4 main.c
  *
  *   Console based interface for Rios using librioutil
@@ -73,6 +73,7 @@ static void progress_no_tty(int x, int X, void *ptr);
 static void new_printfiles(int mflag, int mem_unit);
 void print_info(rio_info_t *info, int type, int mflag, int mem_unit);
 void enter_shell(rios_t *rio, int mflag, int mem_unit);
+int create_playlist (rios_t *rio, int argc, char *argv[]);
 
 #define max(a, b) ((a > b) ? a : b)
 
@@ -123,7 +124,9 @@ int main (int argc, char *argv[]) {
   int total, i, j;
   long int ticks, ttime;
   int lflag = 0, hflag = 0, iflag = 0, vflag = 0, fflag = 0, cflag = 0;
+  int jflag = 0;
   rio_info_t     *info;
+
   int recovery = 0;
 
   struct stat statinfo;
@@ -135,6 +138,10 @@ int main (int argc, char *argv[]) {
     {"delete",  1, 0, 'd'},
     {"debug",   0, 0, 'e'},
     {"format",  0, 0, 'f'},
+    {"shell",   0, 0, 'g'},
+    {"help",    0, 0, 'h'},
+    {"info",    0, 0, 'i'},
+    {"playlist", 0, 0, 'j'},
     {"update",  1, 0, 'u'},
     {"artist",  1, 0, 's'},
     {"title" ,  1, 0, 't'},
@@ -144,9 +151,6 @@ int main (int argc, char *argv[]) {
     {"device",  1, 0, 'o'},
     {"pipe",    0, 0, 'p'},
     {"list",    0, 0, 'l'},
-    {"help",    0, 0, 'h'},
-    {"info",    0, 0, 'i'},
-    {"shell",   0, 0, 'g'},
     {"version", 0, 0, 'v'},
     {"recovery",0, 0, 'z'}
   };
@@ -163,7 +167,7 @@ int main (int argc, char *argv[]) {
   */
   is_a_tty = isatty(1);
 
-  while((c = getopt_long(argc, argv, "W;a:bgld:ec:u:s:t:r:m:p:o:n:fh?ivgz",
+  while((c = getopt_long(argc, argv, "W;a:bgld:ec:u:s:t:r:m:p:o:n:fh?ivgzj",
 			 long_options, &option_index)) != -1){
     switch(c){
     case 'a':
@@ -182,9 +186,6 @@ int main (int argc, char *argv[]) {
     case 'b':
       bflag = 1;
       
-      if (gflag && elvl)
-	fprintf(stderr, "sigh...\n");
-      
       break;
     case 'g':
       gflag = 1;
@@ -198,9 +199,6 @@ int main (int argc, char *argv[]) {
       nflag = 1;
       name = optarg;
       
-      if (gflag && elvl)
-	fprintf(stderr, "sigh...\n");
-      
       break;
     case 'v':
       vflag = 1;
@@ -263,6 +261,10 @@ int main (int argc, char *argv[]) {
     case 'i':
       iflag = 1;
 
+      break;
+    case 'j':
+      jflag = 1;
+
       break;
     case 'z':
       recovery = 1;
@@ -284,7 +286,8 @@ int main (int argc, char *argv[]) {
   }
 
   if (!gflag && !bflag && !aflag && !dflag && !uflag &&
-      !fflag && !iflag && !lflag && !nflag && !cflag && !pipeu)
+      !fflag && !iflag && !lflag && !nflag && !cflag && 
+      !pipeu && !jflag)
       usage();
 
   if (recovery && (!fflag || !uflag) && (dflag || aflag || iflag || lflag ||
@@ -293,6 +296,13 @@ int main (int argc, char *argv[]) {
     exit(1);
   }
 
+  if (jflag && (gflag || bflag || aflag || dflag || uflag ||
+		fflag || iflag || lflag || nflag || cflag ||
+		pipeu)) {
+    fprintf (stderr, "Playlist creation mode cannot take any additional flags.\n");
+    exit (1);
+  }
+  
   if (!recovery)
     fprintf (stderr, "Attempting to open Rio and retrieve song list....");
   else
@@ -309,6 +319,9 @@ int main (int argc, char *argv[]) {
   }
   
   fprintf (stderr, "done\n");
+
+  if (jflag)
+    return create_playlist (current_rio, argc, argv);
       
   if (mflag) {
     if (mem_unit >= return_mem_units_rio (current_rio)) {
@@ -771,13 +784,13 @@ static void new_printfiles(int mflag, int mem_unit) {
   minutes_width = intwidth(max_time / 60);
   minutes_width++;
   
-  header_width = printf("%*s | %*s |  %*s   %*s:%2s %*s\n",
+  header_width = printf("%*s | %*s |  %*s   %*s:%2s %*s %s\n",
 			id_width,
 			"id",
 			max_title_width, "Title",
 			max_name_width, "Name",
 			minutes_width, "mm", "ss",
-			size_width, "Size");
+			size_width, "Size", "Bitrate");
   
   for (j = 0; j < header_width; ++j)
     putchar('-');
@@ -795,13 +808,13 @@ static void new_printfiles(int mflag, int mem_unit) {
 
     
     for (tmpf = flst[j]; tmpf ; tmpf = tmpf->next) {
-      printf("%*i | %*s |  %*s | %*i:%02i %*i\n",
+      printf("%*i | %*s |  %*s | %*i:%02i %*i %*i\n",
 	     id_width, tmpf->num,
 	     max_title_width, tmpf->title,
 	     max_name_width, tmpf->name,
 	     minutes_width, (tmpf->time / 60),
 	     (tmpf->time % 60),
-	     size_width, tmpf->size / 1024);
+	     size_width, tmpf->size / 1024, 7, tmpf->bitrate);
     }
     
     free_file_list (flst[j]);
@@ -832,6 +845,8 @@ void usage(void){
 
 
   printf(" other commands:\n");
+  printf("  -j, --playlist <name> <list of mem_unit,song> create playlist (S-Series and newer)\n");
+  printf("       i.e. rioutil -j fubar 0,0 1,0     (song 0 on mem_unit 0, song 0 on mem_unit 1)\n");
   printf("  -i, --info             rio info\n");
   printf("  -l, --list             list tracks\n");
   printf("  -f, --format           format rio memory (default is internal)\n");
@@ -1138,3 +1153,37 @@ void enter_shell(rios_t *rio, int mflag, int mem_unit){
 
     exit(0);
 }
+
+int create_playlist (rios_t *rio, int argc, char *argv[]) {
+  int nsongs = argc - optind - 1;
+  int i, ret;
+  int *mems, *songs;
+
+  if (nsongs < 1) {
+    fprintf (stderr, "No songs to add!\n");
+    return 1;
+  }
+
+  fprintf (stderr, "Creating playlist %s on Rio.\n", argv[optind]);
+
+  mems = calloc (sizeof (int), nsongs);
+  songs = calloc (sizeof (int), nsongs);
+  
+  optind++;
+
+  for (i = 0 ; i < nsongs ; i++) {
+    sscanf (argv[optind+i], "%d,%d", &mems[i], &songs[i]);
+  }
+
+  ret = create_playlist_rio (rio, argv[optind-1], songs, mems, nsongs);
+
+  if (ret < 0)
+    fprintf (stderr, "Could not create playlist!\n");
+  else
+    fprintf (stderr, "Playlist creation successfull.\n");
+
+  free (mems);
+  free (songs);
+
+  return 0;
+}
